
# Copyright 2003,2004 Frederick Dean

use strict;

package FKong::record;

sub do_comment_keywords
{
   my($recId,$fktable) = @_;
   my $tablesql = $$fktable{'tablesql'};
   if(! $$fktable{'comments'}) {
      $FKong::cgi::keyword{'old_comments'} = "<i>No ${tablesql}_comment table</i>\n";
      $FKong::cgi::keyword{'add-comment'} = "";  # so the template is not expanded
      return;
   };
   my $comments = "";

   my $dbh = FKong::db::SendSQL("SELECT commentId, comment.createTS, format, subject, body, \n".
               "user.realname, COALESCE(user.username,comment.create_by)\n".
               "FROM ${tablesql}_comment AS comment\n".
               "LEFT JOIN user ON (comment.create_by = user.userid)\n".
               "WHERE comment.$$fktable{'pkey'} = $recId\n".
               "ORDER BY createTS");
   while(my($commentId,$when,$format,$subject,$body,$who_real,$who_user) = $dbh->fetchrow_array()) {
      $FKong::cgi::keyword{'commentId'} = $commentId;
      $FKong::cgi::keyword{'subject'} = FKong::cgi::htmlQuote($subject);
      $FKong::cgi::keyword{'body'} = FKong::cgi::htmlQuoteForPre($body);
      FKong::cgi::Linkify($FKong::cgi::keyword{'body'});
      $FKong::cgi::keyword{'when'} = Date::Format::time2str($FKong::config{'TimeFormat'},$when);
      if($FKong::config{'ShowRealnames'} && defined $who_real && $who_real =~ /\S/) {  # if realname not blank
         $FKong::cgi::keyword{'who'} = FKong::cgi::htmlQuote("$who_real ($who_user)");
      } else {  # else show just the username
         $FKong::cgi::keyword{'who'} = FKong::cgi::htmlQuote($who_user);
      }
      $comments .= FKong::cgi::return_expanded_template("template/comment-template.html");
   };
   $FKong::cgi::keyword{'old_comments'} = $comments;
}

sub translate_to_username
{
   my($value,$cacheref) = @_;
   return $value if ! $value;
   return $$cacheref{$value} if $$cacheref{$value};  # check cache before database
   my $dbh = FKong::db::SendSQL("SELECT username FROM user WHERE userid = ". FKong::db::SqlQuote($value));
   my($name) = $dbh->fetchrow_array();
   $name ||= $value;  # use numeric value if username not found
   $$cacheref{$value} = $name;  # save in cache
   return $name;
}

my %length_of_type = ( oneline => 60, integer => 12, hexint => 12, money => 12, unix_ts => 30, user => 30, float => 12 );

sub unixTimestamp {
   return "" if ! $_[0];  # if zero or undefined
   return Date::Format::time2str($FKong::config{'TimeFormat'},$_[0]);
}
sub yes_no { return $_[0] ? 'yes' : 'no'; }
our %type_func = ( unix_ts => \&unixTimestamp, hexint => \&FKong::table::hex_format, bool => \&yes_no,
                   money => \&FKong::table::money_format, user => \&translate_to_username );
my %col_span = ( multiline => 3, oneline => 3, user => 3);

sub show_fields
{
   my($template,$hashref,$andwhere,$fktable) = @_;
   my $lh = $FKong::cgi::form{'lh'} ? ', long_help' : '';  # show long help inline?
   my $dbh = FKong::db::SendSQL("SELECT fieldId, fieldName, sqlName, read_only, type, mult_choices, deflt, short_help, height, width $lh\n".
               "FROM field_def\n".
               "WHERE tablesql = ". FKong::db::SqlQuote($$fktable{'tablesql'}) ."\n". $andwhere .
               "ORDER BY sort, type, fieldName");
   my $form = "";
   my %usercache;
   while(my($fieldId,$fieldName,$sqlName,$read_only,$type,$mult_choices,$default,
         $short_help,$height,$width,$long_help) = $dbh->fetchrow_array()) {  # for each possible field
      my $value = $$hashref{$sqlName};
      $value = $default if ! defined $value;
      $read_only ||= ($FKong::cgi::keyword{'userid'} >= 100 && ! FKong::session::has_priv('edit_features'));  # if they are logged in and cannot edit
      $read_only ||= ($$hashref{'internal'} && $sqlName eq 'url');  # don't chage the url of internal tables
      $read_only ||= ($$hashref{'internal'} && $sqlName eq 'down_maint');  # don't make internal tables down for maintenance
      $FKong::cgi::keyword{$sqlName} = FKong::cgi::htmlQuote($value);  # for custom templates
      my $edit = $FKong::cgi::form{'edit'} ? 
               "<a href=\"$FKong::cgi::keyword{'fkroot'}$$fktable{'url'}/field/edit.html?fieldId=$fieldId\"><font color=\"#669966\" size=-1>E</font></a> " : "";
      if($type eq 'constant') {
         $form .= "<tr><td class=t colspan=4>$edit". ($default || $fieldName) ."</td></tr>\n";
      } elsif($read_only) {
         $value = &{$type_func{$type}}($value,\%usercache) if $type_func{$type};
         $value = FKong::cgi::htmlQuote($value);
         $value = FKong::cgi::Linkify($value);  # linkify
         my $colspan = $col_span{$type} || 1;
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td colspan=$colspan><div class=read_only>$value</div></td>\n <td colspan=2>$short_help</td></tr>\n";
      } elsif($type eq "mult_choice") {
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  "    <td><select name=\"$sqlName\">\n";
         foreach (split(/^/,$mult_choices)) {
            chomp($_);
            my $selected = $value eq $_ ? "SELECTED" : "";
            $form .= "      <option value=\"". FKong::cgi::value_quote($_) ."\" $selected>". FKong::cgi::htmlQuote($_) ."</option>\n";
         }
         $form .= "      </select></td>\n".
                  " <td>$short_help</td></tr>\n";
      } elsif($type eq "bool") {
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td><input name=\"$sqlName\" type=checkbox ". ($value?"CHECKED":"") ."></td>\n".
                  " <td colspan=2>$short_help</td></tr>\n";
      } elsif($type eq "oneline") {
         my $link = '';
         if($value =~ m/(https?:\/\/\S+)/) {  # if field contains a link
            $link = "<a href=\"$1\">Link</a> ";
         } elsif ($value =~ m/([\w\-\.]+\.[a-zA-Z\-\.]+\/[\/\w\.\-\%\?\=\&]*)\s*$/i) {  # or probably a hostname with path
            $link = "<a href=\"http://$1\">Link</a> ";
         } elsif ($value =~ m/([\w\-\.]+\.[a-zA-Z\-\.]+)\s*$/i) {  # or probably a hostname
            $link = "<a href=\"http://$1\/\">Link</a> ";
         }
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td colspan=3><input name=\"$sqlName\"". ($width ? " size=$width" :'') ." value=\"". FKong::cgi::value_quote($value) ."\"> $link</td>\n".
                  " <td>$short_help</td></tr>\n";
      } elsif($type eq "user") {
         $value = $FKong::cgi::keyword{'userid'} if ! $value;  # use ourselves if the defailt is not a real user.
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td colspan=3>";
         if($FKong::config{'UserPulldown'}) {
            $form .= FKong::user::user_pulldown($sqlName,$value);
         } else {
            $value = translate_to_username($value,\%usercache);
            $form .= "<input name=\"$sqlName\"". ($width ? " size=$width" :"") ." value=\"". FKong::cgi::value_quote($value) ."\">";
         };
         $form .= "</td>\n<td>$short_help</td></tr>\n";
      } elsif($type eq "multiline") {
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td colspan=3><textarea WRAP=HARD NAME=\"$sqlName\"". ($height ? " ROWS=$height" :'') .
                  ($width ? " COLS=$width" :'') .">". 
                  FKong::cgi::htmlQuoteForPre($value) ."</textarea></td></tr>\n";
         $FKong::cgi::keyword{$sqlName} = FKong::cgi::htmlQuoteForPre($value); # for custom templates
      } else {
         $form .= "<tr><td class=t>$edit" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td><input name=\"$sqlName\"". ($width ? " size=$width":'') ." value=\"". 
                  FKong::cgi::htmlQuote( $type_func{$type} ? &{$type_func{$type}}($value) : $value ) ."\"></td>\n".
                  " <td>$short_help</td></tr>\n";
      };
      $form .= "<tr><td class=h colspan=4>$long_help</td></tr>\n" if $lh;  # append long help
   };
   $FKong::cgi::keyword{'fform'} = $form;
   FKong::cgi::print_expanded_template($template);
   $FKong::cgi::keyword{'old_comments'} = "-- deleted in ". __FILE__ ." line ". __LINE__ ." --";  # so the debugging info is not too big
}

$FKong::table_func{'batch_row.html'} = \&show_batch_row;

sub show_batch_row
{
   my($fktable) = @_;

   # check that they have permission 
   FKong::session::must_have_priv("view_features");
   FKong::session::must_have_priv("view_fields") if $$fktable{'internal'};
   $FKong::cgi::keyword{'formurl'} = "submit.html"; 
   my $recs = FKong::session::get_state("$$fktable{'tablesql'}-list");
   if($recs) {
      $FKong::cgi::keyword{'hidden'} .= "";
      $FKong::cgi::keyword{'records'} = join(" &nbsp; \n",map { "<input type=checkbox name=f$_ CHECKED><a href=\"$$fktable{'recprefix'}$_.html\">".
                                                       "$$fktable{'recprefix'}$_</a>" } split(/,/, $recs));
   } else {
      $FKong::cgi::keyword{'hidden'} .= "<input type=hidden name=ids value=all>\n";
      $FKong::cgi::keyword{'records'} = "<font color=red>all records (no commenting)</font>"
   };
   my $lh = $FKong::cgi::form{'lh'} ? ', long_help' : '';  # show long help inline?
   my $dbh = FKong::db::SendSQL("SELECT fieldName, sqlName, type, mult_choices, deflt, short_help, width, height $lh\n".
               "FROM field_def\n".
               "WHERE batch AND tablesql = ". FKong::db::SqlQuote($$fktable{'tablesql'}) ."\n".
               "ORDER BY sort, type, fieldName");
   my $form = "";
   while(my($fieldName,$sqlName,$type,$mult_choices,$default,$short_help,$width,$height,$long_help) = $dbh->fetchrow_array()) {  # for each possible field
      if($type eq "mult_choice") {
         $form .= "<tr><td class=t>" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  "    <td><select name=\"$sqlName\">\n";
         $form .= "      <option value=\"\" SELECTED>*** No Change ***</option>\n";
         foreach (split(/^/,$mult_choices)) {
            chomp($_);
            $form .= "      <option value=\"". FKong::cgi::value_quote($_) ."\">". FKong::cgi::htmlQuote($_) ."</option>\n";
         }
         $form .= "      </select></td>\n".
                  " <td>$short_help</td></tr>\n";
      } elsif($type eq "multiline") {
         $form .= "<tr><td class=t>" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td colspan=3><textarea WRAP=HARD NAME=\"$sqlName\"". ($height ? " ROWS=$height" :"") . 
                  ($width ? " COLS=$width" :"") ."></textarea></td></tr>\n";
      } elsif($type eq 'constant') {
         $form .= "<tr><td class=t colspan=4>". ($default || $fieldName) ."</td></tr>\n";
      } elsif($type eq 'bool') {
         $form .= "<tr><td class=t>" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td><input name=\"$sqlName\" type=radio value=1> Yes \n".
                  "<input name=\"$sqlName\" type=radio value=0> No \n".
                  "<input name=\"$sqlName\" type=radio value=\"\" CHECKED> Unchanged</td></tr>\n";
      } elsif($type eq 'user' && $FKong::config{'UserPulldown'}) {
         $form .= "<tr><td class=t>" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td>".  FKong::user::user_pulldown($sqlName,0,1) ."</td>\n".
                  " <td>$short_help</td></tr>\n";
      } else {
         $form .= "<tr><td class=t>" . FKong::cgi::htmlQuote($fieldName) ."</td>\n". 
                  " <td><input name=\"$sqlName\"". ($width ? " size=$width" :'') ."></td>\n".
                  " <td>$short_help</td></tr>\n";
      };
      $form .= "<tr><td class=h colspan=4>$long_help</td></tr>\n" if $lh;  # append long help
   }
   $FKong::cgi::keyword{'fform'} = $form;
   FKong::cgi::print_expanded_template("template/batch_row.html");
}

# Note: the fktable table does not use this code because $FKong::url_func{"fktable/new"} is defined
# Note: the user table does not use this code because $FKong::url_func{"user/new"} is defined
sub show_new_form
{
   my($fktable) = @_;
   my $tablesql  = $$fktable{'tablesql'};
   # check that they have permission 
   FKong::session::must_have_priv("edit_features");
   if($$fktable{'internal'}) {
      FKong::Fatal("Sorry. You cannot edit internal tables with this form.");
   };
   $FKong::cgi::keyword{'formurl'} = FKong::cgi::Uri("/$$fktable{'url'}/submit.html");  # important so the redirect doesn't trash this page.
   my $hashref = { };
   if($FKong::cgi::form{'duplicate'} && $FKong::cgi::form{'duplicate'} =~ /(\d+)/ ) {
      my $dbh2 = FKong::db::SendSQL("SELECT * FROM $tablesql WHERE $$fktable{'pkey'} = $1");
      $hashref = $dbh2->fetchrow_hashref() || {};
   };
   my $dbh = FKong::db::SendSQL("SELECT sqlName, deflt, rset_on_dup FROM field_def WHERE tablesql = ". FKong::db::SqlQuote($tablesql));
   while(my($sqlName,$default,$rset_on_dup) = $dbh->fetchrow_array()) {
      $$hashref{$sqlName} = $default if ! defined $$hashref{$sqlName} || $rset_on_dup;
   }
   my $template = FKong::cgi::first_existing("template/$tablesql-new.html","template/feature-new-template.html");
   show_fields($template,$hashref,"AND on_new_form\n",$fktable);
}

$FKong::table_func{'new.html'} = \&show_new_form;

sub find_index
{
   my $pattern = shift;
   my $index = 0;
   foreach (@_) { return $index if $_ eq $pattern; $index++ }
   return undef;
}


#  This form is tricky because sometimes it can be seen from two URLs, both with and without "$$fktable{'url'}/"
sub show_edit_form
{
   my($recId,$fktable) = @_;

   my $recprefix = $$fktable{'recprefix'};
   my $tablesql = $$fktable{'tablesql'};
   my $pkey = $$fktable{'pkey'};
   # check that they have permission 
   FKong::session::must_have_priv("view_features");
   FKong::session::must_have_priv("view_fields") if $$fktable{'internal'};
   my $feature_name = "$recprefix$recId"; 
   $FKong::cgi::keyword{'feature_name'} = $feature_name; 
   my $dbh = FKong::db::SendSQL("SELECT * FROM $tablesql WHERE $pkey = $recId");
   my $hashref = $dbh->fetchrow_hashref();
   if(! defined $hashref) { 
      FKong::Fatal("Feature ". FKong::cgi::htmlQuote($feature_name) ." does not exist."); 
      return;
   }
   my $modifyTS = $$hashref{'modifyTS'} || 1;
   $$hashref{'changeId'} ||= 0;  # ensure defined (fake out changeId if ${tablesql}_change table not present
   $FKong::cgi::keyword{'hidden'} .= "<input type=hidden name=changeId value=$$hashref{'changeId'}>\n";
   $FKong::cgi::keyword{'hidden'} .= "<input type=hidden name=feature_id value=$$hashref{$pkey}>\n".
                                     "<input type=hidden name=modifyTS value=$modifyTS>";  # to detect mid-air collisions
   $FKong::cgi::keyword{'formurl'} = FKong::cgi::Uri("/$$fktable{'url'}/submit.html");  # important so the redirect doesn't trash this page.
   # format list links
   my($list,$query) = FKong::session::get_state("features-list","$tablesql-query");
   if($list) {
      my @ids = split(/,/,$list);
      my $index = find_index($recId,@ids);
      if(defined $index) {  # if current is in the list
         $FKong::cgi::keyword{'list_links'} .= "(". ($index+1) ." of ". scalar(@ids) .") &nbsp; \n";
      } else {  # else current is not in the list
         $FKong::cgi::keyword{'list_links'} .= "(not in list) &nbsp; \n";
      };
      $FKong::cgi::keyword{'list_links'} .= "<a href=\"". FKong::cgi::value_quote("$recprefix$ids[0].html") .
                                                   "\">First</a> &nbsp; \n";
      if(defined $index) {  # if current is in the list
         $FKong::cgi::keyword{'list_links'} .= "<a href=\"". FKong::cgi::value_quote("$recprefix$ids[$index-1].html") .
                                                      "\">Previous</a> &nbsp; \n";
         if($index == $#ids) {  # if current is last
            $FKong::cgi::keyword{'list_links'} .= "<a href=\"". FKong::cgi::value_quote("$recprefix$ids[0].html") .
                                                        "\">Next</a> &nbsp; \n";
         } else {  # else current is not last
            $FKong::cgi::keyword{'list_links'} .= "<a href=\"". FKong::cgi::value_quote("$recprefix$ids[$index+1].html") .
                                                        "\">Next</a> &nbsp; \n";
         };
      } else {  # else not in list
         $FKong::cgi::keyword{'list_links'} .= "Previous &nbsp; Next &nbsp; \n";
      }
      $FKong::cgi::keyword{'list_links'} .= "<a href=\"". FKong::cgi::value_quote("$recprefix$ids[-1].html") ."\">Last</a> &nbsp; \n";
      $FKong::cgi::keyword{'list_links'} .= " &nbsp; &nbsp; <a href=list.html>List</a> \n";
   } else {  # else there is no current list
      $FKong::cgi::keyword{'list_links'} .= "<font color=#888888>(no current list)</font> &nbsp; \n";
   }
   $query ||= "";  # ensure defined
   $FKong::cgi::keyword{'list_links'} .= " &nbsp; &nbsp; <a href=$FKong::cgi::keyword{'fkroot'}$$fktable{'url'}/search.html?$query>Query</a> \n";
   $FKong::cgi::keyword{'delete'} = FKong::session::has_priv('delete_features') ? 
                             " &nbsp; ". FKong::cgi::Link("/delete.html",["id=$recId"]) ."delete</a>" : "";
   do_comment_keywords($$hashref{$pkey},$fktable);
   my $template = FKong::cgi::first_existing("template/$tablesql-edit.html","template/feature-template.html");
   # Finally, show the damn thing
   show_fields($template,$hashref,"AND on_edit_form ",$fktable);
}


sub delete_confirm
{
   my($fktable) = @_;
   my $recprefix = $$fktable{'recprefix'};
   my $tablesql = $$fktable{'tablesql'};
   my $pkey = $$fktable{'pkey'};
   FKong::Fatal("Sorry. You cannot delete from internal tables with this form.") if $$fktable{'internal'};
   FKong::session::must_have_priv("view_features");
   FKong::session::must_have_priv('delete_features');
   FKong::Fatal("Internal Error: delete has no id") if ! $FKong::cgi::form{'id'};
   FKong::Fatal("Internal Error: delete has malformed id") if $FKong::cgi::form{'id'} !~ /^\d+$/;
   $FKong::cgi::keyword{'hidden'} = qq(<input type=hidden name=id value=$FKong::cgi::form{'id'}>\n);
   my $dbh = FKong::db::SendSQL("SELECT $tablesql.$pkey\n".
                         "FROM $tablesql \n".
                         "WHERE $tablesql.$pkey = ". FKong::db::SqlQuote($FKong::cgi::form{'id'}) ." GROUP BY $tablesql.$pkey");
   my($id,$company,$owner,$contact,$num_license) = $dbh->fetchrow_array();
   FKong::Fatal("Internal Error: feature does not exist") if ! $id;
   $FKong::cgi::keyword{'featureName'} = FKong::cgi::Link("/$recprefix$id.html") ."$recprefix$id</a>";
   $FKong::cgi::keyword{'fieldRows'} = "";
   $FKong::cgi::keyword{'formurl'} = "list.html";
   FKong::cgi::print_expanded_template("template/delete_confirm-template.html");
}

$FKong::table_func{'delete.html'} = \&delete_confirm;


1;

